/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mithrim.epf.core;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;

import org.mithrim.epf.core.common.OutputTerminal;

/**
 * An <code>EventProducer</code> is an entity at the edge of an event processing system
 * that introduces events into the system.
 *
 * @author Dirk Jablonski
 */
public interface EventProducer {
	/**
	 * Return the id of this event producer instance.
	 *
	 * @return the id of this instance
	 */
	String getId();

	/**
	 * Sets the {link OutputTerminal}s to send event from.
	 * 
	 * @param outputTerminals the {link OutputTerminal}s for this producer
	 */
	void setOutputTerminals(Set<OutputTerminal> outputTerminals);

	/**
	 * Produce an event of the {link EventType} identified by the given event type id
	 * without any payload attributes.
	 *
	 * @param eventTypeId the event type id of the event to be created
	 *
	 * @return <code>true</code>, if the event has been successfully created and has been
	 *         sent to at least one channel, <code>false</code> otherwise.
	 */
	boolean produceEvent(String eventTypeId);

	/**
	 * Produce an event of the {link EventType} identified by the given event type id,
	 * with a single payload attribute with the given name and value.
	 *
	 * @param eventTypeId the event type id of the event to be created
	 * @param attributeName the name of the payload attribute to set
	 * @param attributeValue the value for the payload attribute

	 * @return <code>true</code>, if the event has been successfully created and has been
	 *         sent to at least one channel, <code>false</code> otherwise.
	 */
	boolean produceEvent(String eventTypeId, String attributeName, Serializable attributeValue);

	/**
	 * Produce an event of the {link EventType} identified by the given event type id,
	 * with payload attributes given as entries if the <code>attribues</code> parameter.
	 *
	 * @param eventTypeId the event type id of the event to be created
	 * @param attributes the map containing the payload attributes to set on the event
	 *
	 * @return <code>true</code>, if the event has been successfully created and has been
	 *         sent to at least one channel, <code>false</code> otherwise.
	 */
	boolean produceEvent(String eventTypeId, Map<String, Serializable> attributes);

	/**
	 * Produce an event of the {link EventType} identified by the given event type id,
	 * with payload attributes given as entries if the <code>attribues</code> parameter,
	 * and additionally attaches the serializable open content to the event.
	 *
	 * @param eventTypeId the event type id of the event to be created
	 * @param attributes the map containing the payload attributes to set on the event
	 * @param openContent the open content to attach to the event
	 *
	 * @return <code>true</code>, if the event has been successfully created and has been
	 *         sent to at least one channel, <code>false</code> otherwise.
	 */
	boolean produceEvent(String eventTypeId, Map<String, Serializable> attributes, Serializable openContent);
}
