/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mithrim.epf.core;

import java.util.Set;

import org.mithrim.epf.core.common.AttributeType;
import org.mithrim.epf.core.common.Occurrence;
import org.mithrim.epf.core.common.TemporalGranularity;

/**
 * An <code>EventType</code> is a specification for a set of event objects that have
 * the same semantic intent and same structure; every event object is considered to
 * be an instance of an event type.
 *
 * @author Dirk Jablonski
 */
public class EventType {
	/**
	 * Default value for compositionFlag.
	 */
	public static final Boolean DEFAULT_COMPOSITION_FLAG = Boolean.FALSE;

	/**
	 * Default value for temporalGranularity.
	 */
	public static final TemporalGranularity DEFAULT_TEMPORAL_GRANULARITY = TemporalGranularity.MILLISECOND;

	/**
	 * Default value for occurrenceTimeIndicator.
	 */
	public static final Occurrence DEFAULT_OCCURRENCE_TIME_INDICATOR = Occurrence.OPTIONAL;

	/**
	 * Default value for certaintyIndicator.
	 */
	public static final Occurrence DEFAULT_CERTAINTY_INDICATOR = Occurrence.NOT_APPLICABLE;

	/**
	 * Default value for annotationIndicator.
	 */
	public static final Occurrence DEFAULT_ANNOTATION_INDICATOR = Occurrence.OPTIONAL;

	/**
	 * Default value for sourceIndicator.
	 */
	public static final Occurrence DEFAULT_SOURCE_INDICATOR = Occurrence.REQUIRED;

	/**
	 * Default value for openContentIndicator.
	 */
	public static final Occurrence DEFAULT_OPEN_CONTENT_INDICATOR = Occurrence.NOT_APPLICABLE;

	/**
	 * Default value for abstract property.
	 */
	public static final boolean DEFAULT_ABSTRACT_FLAG = false;

	/**
	 * The event type id uniquely identifies the type.
	 */
	private String id;

	/**
	 * The event composition flag denotes whether the event type is the
	 * composition of other events or not.
	 */
	private Boolean compositionFlag = DEFAULT_COMPOSITION_FLAG;

	/**
	 * The temporal granularity defines the maximum accuracy of timestamps (i.e.
	 * creationTime and detectionTime)
	 */
	private TemporalGranularity temporalGranularity = DEFAULT_TEMPORAL_GRANULARITY;

	/**
	 * Indicates, whether the <code>occuranceTime</code> attribute must, may, or
	 * must not appear in an event instance.
	 */
	private Occurrence occurrenceTimeIndicator = DEFAULT_OCCURRENCE_TIME_INDICATOR;

	/**
	 * Indicates, whether the <code>certainty</code> attribute must, may, or
	 * must not appear in an event instance.
	 */
	private Occurrence certaintyIndicator = DEFAULT_CERTAINTY_INDICATOR;

	/**
	 * Indicates, whether the <code>annotation</code> attribute must, may, or
	 * must not appear in an event instance.
	 */
	private Occurrence annotationIndicator = DEFAULT_ANNOTATION_INDICATOR;

	/**
	 * Indicates, whether the <code>source</code> attribute must, may, or must
	 * not appear in an event instance.
	 */
	private Occurrence sourceIndicator = DEFAULT_SOURCE_INDICATOR;

	private Set<AttributeType> payloadAttributeTypes;

	/**
	 * Indicates, whether open content must, may, or must not appear in an event
	 * instance.
	 */
	private Occurrence openContentIndicator = DEFAULT_OPEN_CONTENT_INDICATOR;

	/**
	 * Determines if this event type is abstract, and therefore must not be used
	 * for an actual event instance.
	 */
	private boolean abstractType = DEFAULT_ABSTRACT_FLAG;

	/**
	 * If set, holds the id of the parent type in the inheritance hierarchy.
	 */
	private String parentTypeId;

	/**
	 * @return the id
	 */
	public String getId() {
		return this.id;
	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setId(final String id) {
		this.id = id;
	}

	/**
	 * @return the compositionFlag
	 */
	public Boolean getCompositionFlag() {
		return compositionFlag;
	}

	/**
	 * @param compositionFlag
	 *            the compositionFlag to set
	 */
	public void setCompositionFlag(final Boolean compositionFlag) {
		this.compositionFlag = compositionFlag;
	}

	/**
	 * @return the temporalGranularity
	 */
	public TemporalGranularity getTemporalGranularity() {
		return temporalGranularity;
	}

	/**
	 * @param temporalGranularity
	 *            the temporalGranularity to set
	 */
	public void setTemporalGranularity(final TemporalGranularity temporalGranularity) {
		this.temporalGranularity = temporalGranularity;
	}

	/**
	 * @return the occurrenceTimeIndicator
	 */
	public Occurrence getOccurrenceTimeIndicator() {
		return occurrenceTimeIndicator;
	}

	/**
	 * @param occurrenceTimeIndicator
	 *            the occuranceTimeIndicator to set
	 */
	public void setOccurrenceTimeIndicator(final Occurrence occurrenceTimeIndicator) {
		this.occurrenceTimeIndicator = occurrenceTimeIndicator;
	}

	/**
	 * @return the certaintyIndicator
	 */
	public Occurrence getCertaintyIndicator() {
		return certaintyIndicator;
	}

	/**
	 * @param certaintyIndicator
	 *            the certaintyIndicator to set
	 */
	public void setCertaintyIndicator(final Occurrence certaintyIndicator) {
		this.certaintyIndicator = certaintyIndicator;
	}

	/**
	 * @return the annotationIndicator
	 */
	public Occurrence getAnnotationIndicator() {
		return annotationIndicator;
	}

	/**
	 * @param annotationIndicator
	 *            the annotationIndicator to set
	 */
	public void setAnnotationIndicator(final Occurrence annotationIndicator) {
		this.annotationIndicator = annotationIndicator;
	}

	/**
	 * @return the sourceIndicator
	 */
	public Occurrence getSourceIndicator() {
		return sourceIndicator;
	}

	/**
	 * @param sourceIndicator
	 *            the sourceIndicator to set
	 */
	public void setSourceIndicator(final Occurrence sourceIndicator) {
		this.sourceIndicator = sourceIndicator;
	}

	/**
	 * @return the payloadAttributes
	 */
	public Set<AttributeType> getPayloadAttributeTypes() {
		return payloadAttributeTypes;
	}

	/**
	 * @param payloadAttributeTypes
	 *            the payloadAttributeTypes to set
	 */
	public void setPayloadAttributeTypes(final Set<AttributeType> payloadAttributeTypes) {
		this.payloadAttributeTypes = payloadAttributeTypes;
	}

	/**
	 * @return the openContentIndicator
	 */
	public Occurrence getOpenContentIndicator() {
		return openContentIndicator;
	}

	/**
	 * @param openContentIndicator
	 *            the openContentIndicator to set
	 */
	public void setOpenContentIndicator(final Occurrence openContentIndicator) {
		this.openContentIndicator = openContentIndicator;
	}

	/**
	 * @return <code>true</code> if this type is abstract is marked as abstract,
	 *         <code>false</code> otherwise.
	 */
	public boolean isAbstractType() {
		return abstractType;
	}

	/**
	 * @param abstractType
	 *            the abstractType to set
	 */
	public void setAbstractType(final boolean abstractType) {
		this.abstractType = abstractType;
	}

	/**
	 * @return the parentType
	 */
	public String getParentTypeId() {
		return parentTypeId;
	}

	/**
	 * @param parentTypeId
	 *            the parentTypeId to set
	 */
	public void setParentTypeId(final String parentTypeId) {
		this.parentTypeId = parentTypeId;
	}

}
