/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mithrim.epf.core.common;

import java.util.Set;

import org.mithrim.epf.core.EventChannelOutputAdapter;
import org.mithrim.epf.core.consumer.DefaultEventConsumer;
import org.mithrim.epf.core.event.Event;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 *
 * @author Dirk Jablonski
 */
public class DefaultInputTerminal implements InputTerminal {
	/**
	 * The logger for this class.
	 */
	private static final Logger logger = LoggerFactory.getLogger(DefaultEventConsumer.class);

	/**
	 * The identifier of this input terminal.
	 */
	private String id;

	/**
	 * The sources to receive events from.
	 */
	private Set<EventChannelOutputAdapter> sources;

	/**
	 * The {link EventHandler} to process events from this input terminal.
	 */
	private EventHandler handler;

	/**
	 * The list of {link EventType} IDs to process events from this input terminal.
	 */
	private Set<String> acceptedEventTypeIds;

	@Override
	public void receive(final Event event) {
		if (null != acceptedEventTypeIds && acceptedEventTypeIds.contains(event.getTypeId())) {
			if (null != handler) {
				handler.handleEvent(event);
			}
			else {
				logger.warn("Input terminal {} has no EventHandler set, discarding accepted event {}", getId(), event.getId());
			}
		}
		else {
			logger.trace("Event {} does not match any accepted event type of input terminal {}, ignoring", event.getId(), getId());
		}
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.common.InputTerminal#getId()
	 */
	@Override
	public String getId() {
		return id;
	}

	public void setId(final String id) {
		this.id = id;
	}

	protected EventHandler getEventHandler() {
		return handler;
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.common.InputTerminal#setEventHandler(org.mithrim.epf.core.common.EventHandler)
	 */
	@Override
	public void setEventHandler(final EventHandler handler) {
		this.handler = handler;
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.common.OutputTerminal#isAcceptedType(java.lang.String)
	 */
	@Override
	public boolean isAcceptedType(final String eventTypeId) {
		return (null != acceptedEventTypeIds && acceptedEventTypeIds.contains(eventTypeId));
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.common.InputTerminal#setAcceptedEventTypeIds(java.util.Set)
	 */
	@Override
	public void setAcceptedEventTypeIds(final Set<String> acceptedEventTypeIds) {
		this.acceptedEventTypeIds = acceptedEventTypeIds;
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.common.InputTerminal#setSources(java.util.Set)
	 */
	@Override
	public void setSources(final Set<EventChannelOutputAdapter> sources) {
		this.sources = sources;

		if (null != sources) {
			for (EventChannelOutputAdapter source : sources) {
				source.registerInputTerminal(this);
			}
		}
	}
}
