/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mithrim.epf.core.event;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import org.mithrim.epf.core.common.TemporalGranularity;


/**
 * An <code>Event</code> is the programming entity representing an event occurrence
 * in the application.
 *
 * @author Dirk Jablonski
 */
public final class Event implements Serializable {
	private static final long serialVersionUID = -3161115188717563539L;

	/**
	 * The id of the <code>EventType</code> of this event.
	 */
	private final String typeId;

	/**
	 * The temporal granularity for timestamp header attributes (i.e. creationTime and
	 * detectionTime).
	 */
	private final TemporalGranularity temporalGranularity;

	/**
	 * A system-generated unique ID for this event instance.
	 */
	private final String id;

	/**
	 * The occurrence time of this particular event, with the event type's
	 * temporal granularity.
	 */
	private Long occurrenceTime;

	/**
	 * The certainty of the occurrence of this event.
	 */
	private Float certainty;

	/**
	 * A free-text explanation of what happened in this particular event.
	 */
	private String annotation;

	/**
	 * The system-generated timestamp that records the time at which the event
	 * became known to the event processing system (i.e. when it was created),
	 * with the event type's temporal granularity.
	 */
	private Long detectionTime;

	/**
	 * The name of the entity that originated this event. This can be either an
	 * event producer or an event processing agent.
	 */
	private String source;

	/**
	 * The actual payload attributes for this event.
	 */
	private Map<String, Serializable> payloadAttributes;

	/**
	 * The fully qualified class name of the open content object.
	 */
	private String openContentType;

	/**
	 * The open content object. This must implement the <code>Serializable</code>
	 * interface to allow transfer of the event in distributed environments.
	 */
	private Serializable openContent;


	protected Event(final String typeId, final TemporalGranularity temporalGranularity) {
		if (null == typeId) {
			throw new IllegalArgumentException("typeId must not be null");
		}

		if (null == temporalGranularity) {
			throw new IllegalArgumentException("temporalGranularity must not be null");
		}

		this.typeId = typeId;
		this.temporalGranularity = temporalGranularity;
		this.id = UUID.randomUUID().toString();
	}

	public String getTypeId() {
		return typeId;
	}

	public TemporalGranularity getTemporalGranularity() {
		return temporalGranularity;
	}

	public String getId() {
		return id;
	}

	public Long getOccurrenceTime() {
		return occurrenceTime;
	}

	protected void setOccurrenceTime(final Long occurrenceTime) {
		this.occurrenceTime = occurrenceTime;
	}

	public Float getCertainty() {
		return certainty;
	}

	protected void setCertainty(final Float certainty) {
		this.certainty = certainty;
	}

	public String getAnnotation() {
		return annotation;
	}

	protected void setAnnotation(final String annotation) {
		this.annotation = annotation;
	}

	public Long getDetectionTime() {
		return detectionTime;
	}

	protected void setDetectionTime(final Long detectionTime) {
		this.detectionTime = detectionTime;
	}

	public String getSource() {
		return source;
	}

	protected void setSource(final String source) {
		this.source = source;
	}

	public Map<String, Serializable> getPayloadAttributes() {
		if (null != payloadAttributes) {
			return Collections.unmodifiableMap(payloadAttributes);
		}
		else {
			return null;
		}
	}

	public Serializable getPayloadAttribute(final String attributeName) {
		if (null == payloadAttributes) {
			return null;
		}
		else {
			return payloadAttributes.get(attributeName);
		}
	}

	protected void setPayloadAttributes(final Map<String, Serializable> payloadAttributes) {
		if (null != payloadAttributes) {
			this.payloadAttributes = new HashMap<String, Serializable>(payloadAttributes.size());
			this.payloadAttributes.putAll(payloadAttributes);
		}
	}

	public String getOpenContentType() {
		return openContentType;
	}

	public Serializable getOpenContent() {
		return openContent;
	}

	protected void setOpenContent(final Serializable openContent) {
		if (null != openContent) {
			this.openContent = openContent;
			this.openContentType = openContent.getClass().getName();
		}
	}
}
