/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mithrim.epf.core.event;

import java.io.Serializable;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import org.mithrim.epf.core.EventType;

/**
 *
 *
 * @author Dirk Jablonski
 */
public final class EventBuilder {
    private EventType eventType;
    private Long occurrenceTime;
    private Float certainty;
    private String annotation;
    private Long detectionTime;
    private String source;
    private Map<String, Serializable> payloadAttributes;
    private Serializable openContent;

    public EventBuilder(final EventType eventType) {
        if (null == eventType) {
            throw new IllegalArgumentException("eventType must not be null or empty");
        }

        if (eventType.isAbstractType()) {
            throw new IllegalArgumentException("eventType is declared abstract, instantiation not allowed");
        }

        this.eventType = eventType;
    }

    public EventBuilder setOccurrenceTime(final Long occurrenceTime) {
        this.occurrenceTime = occurrenceTime;

        return this;
    }

    public EventBuilder setCertainty(final Float certainty) {
        this.certainty = certainty;

        return this;
    }

    public EventBuilder setAnnotation(final String annotation) {
        this.annotation = annotation;

        return this;
    }

    public EventBuilder setDetectionTime(final Long detectionTime) {
        this.detectionTime = detectionTime;

        return this;
    }

    public EventBuilder setSource(final String source) {
        this.source = source;

        return this;
    }

    public EventBuilder setPayloadAttribute(final String attributeName, final Serializable attributeValue) {
        if (null == this.payloadAttributes) {
            this.payloadAttributes = new HashMap<String, Serializable>();
        }

        payloadAttributes.put(attributeName, attributeValue);

        return this;
    }

    public EventBuilder setOpenContent(final Serializable openContent) {
        this.openContent = openContent;

        return this;
    }

    public Event build() {
        final Event event = new Event(eventType.getId(), eventType.getTemporalGranularity());

        event.setOccurrenceTime(adjustToGranularity(occurrenceTime));
        event.setCertainty(certainty);
        event.setAnnotation(annotation);

        if (null == detectionTime) {
            event.setDetectionTime(adjustToGranularity(System.currentTimeMillis()));
        }
        else {
            event.setDetectionTime(adjustToGranularity(detectionTime));
        }

        event.setSource(source);
        event.setPayloadAttributes(payloadAttributes);
        event.setOpenContent(openContent);

        return event;
    }

    /**
     * HINT: This method makes explicit use of autoboxing.
     *
     * @param timestampValue
     *            the timestamp value to adjust
     * @return the timestamp value adjusted to the temporal granularity
     */
    private Long adjustToGranularity(final Long timestampValue) {
        if (null == timestampValue) {
            return null;
        }

        final Calendar c = new GregorianCalendar();
        c.setTimeInMillis(timestampValue);

        switch (eventType.getTemporalGranularity()) {
            case SECOND: {
                c.set(Calendar.MILLISECOND, 0);

                return c.getTimeInMillis();
            }
            case MINUTE: {
                c.set(Calendar.MILLISECOND, 0);
                c.set(Calendar.SECOND, 0);

                return c.getTimeInMillis();
            }
            case HOUR: {
                c.set(Calendar.MILLISECOND, 0);
                c.set(Calendar.SECOND, 0);
                c.set(Calendar.MINUTE, 0);

                return c.getTimeInMillis();
            }
            case DAY: {
                c.set(Calendar.MILLISECOND, 0);
                c.set(Calendar.SECOND, 0);
                c.set(Calendar.MINUTE, 0);
                c.set(Calendar.HOUR_OF_DAY, 0);

                return c.getTimeInMillis();
            }
            default: {
                // TemporalGranularity.MILLISECOND
                return c.getTimeInMillis();
            }
        }
    }
}
