/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mithrim.epf.core.producer;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;

import org.mithrim.epf.core.EventProducer;
import org.mithrim.epf.core.common.OutputTerminal;
import org.mithrim.epf.core.event.Event;
import org.mithrim.epf.core.event.EventBuilder;
import org.mithrim.epf.core.event.EventBuilderFactory;
import org.mithrim.epf.core.validation.EventValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This is the default implementation of the {link EventProducer} interface.
 *
 * @author Dirk Jablonski
 */
public class DefaultEventProducer implements EventProducer {
	/**
	 * The logger for this class.
	 */
	private static final Logger logger = LoggerFactory.getLogger(DefaultEventProducer.class);

	/**
	 * The identifier of this producer.
	 */
	private String id;

	/**
	 * The {@link OutputTerminal} to send events to.
	 */
	private Set<OutputTerminal> outputTerminals;

	/**
	 * The {link EventBuilderFactory} to use for event creation.
	 */
	private EventBuilderFactory eventBuilderFactory;

	/**
	 * Flag to enable/disable generation of occurrence time,
	 * if not already contained in attributes.
	 */
	private boolean occurrenceTimeEnabled;

	/**
	 * Flag to enable/disable event validation (if validator is set).
	 */
	private boolean validationEnabled;

	/**
	 * The {link EventValidator} to use for validation, if enabled.
	 */
	private EventValidator validator;

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.EventProducer#produceEvent(java.lang.String)
	 */
	@Override
	public boolean produceEvent(final String eventTypeId) {
		final EventBuilder builder = prepareBasicBuilder(eventTypeId);

		return dispatch(builder.build());
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.EventProducer#produceEvent(java.lang.String, java.lang.String, java.io.Serializable)
	 */
	@Override
	public boolean produceEvent(final String eventTypeId, final String attributeName, final Serializable attributeValue) {
		final EventBuilder builder = prepareBasicBuilder(eventTypeId);

		builder.setPayloadAttribute(attributeName, attributeValue);

		return dispatch(builder.build());
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.EventProducer#produceEvent(java.lang.String, java.util.Map)
	 */
	@Override
	public boolean produceEvent(final String eventTypeId, final Map<String, Serializable> attributes) {
		final EventBuilder builder = prepareBasicBuilder(eventTypeId);

		for (String attributeName : attributes.keySet()) {
			builder.setPayloadAttribute(attributeName, attributes.get(attributeName));
		}

		return dispatch(builder.build());
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.EventProducer#produceEvent(java.lang.String, java.util.Map, java.io.Serializable)
	 */
	@Override
	public boolean produceEvent(final String eventTypeId, final Map<String, Serializable> attributes, final Serializable openContent) {
		final EventBuilder builder = prepareBasicBuilder(eventTypeId);

		for (String attributeName : attributes.keySet()) {
			builder.setPayloadAttribute(attributeName, attributes.get(attributeName));
		}

		builder.setOpenContent(openContent);

		return dispatch(builder.build());
	}

	protected EventBuilder prepareBasicBuilder(final String eventTypeId) {
		final EventBuilder builder = eventBuilderFactory.createEventBuilder(eventTypeId);

		builder.setSource(getId());

		if (isOccurrenceTimeEnabled()) {
			builder.setOccurrenceTime(new Long(System.currentTimeMillis()));
		}

		return builder;
	}

	protected boolean dispatch(final Event event) {
		if (validationEnabled) {
			if (null != validator) {
				if (!validator.isValid(event)) {
					logger.error("Event {} failed validation, does not match event type definition {}", event.getId(), event.getTypeId());
					throw new IllegalArgumentException("Event failed validation, does not match event type definition");
				}
			}
			else {
				logger.warn("Validation enabled for producer {}, but no validator set", getId());
			}
		}

		boolean sendResult = true;

		if (null != getOutputTerminals()) {
			for (OutputTerminal terminal : getOutputTerminals()) {
				if (terminal.isAcceptedType(event.getTypeId())) {
					logger.trace("Sending event {} to output terminal {}", event.getId(), terminal.getId());
					sendResult &= terminal.send(event);
				}
				else {
					logger.trace("Skipping output terminal {} for event {}, type not accepted", terminal.getId(), event.getId());
				}
			}
		}

		return sendResult;
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.EventProducer#getId()
	 */
	@Override
	public String getId() {
		return id;
	}

	/**
	 * Sets the id of this producer instance. It is used as the <code>source</code> header attribute
	 * for event types which have an optional or required source indicator.
	 *
	 * @param id the id for this producer instance
	 */
	public void setId(final String id) {
		this.id = id;
	}

	/**
	 * Returns the {link OutputTerminal}s of this producer instance.
	 *
	 * @return the {link OutputTerminal}s of this producer instance
	 */
	protected Set<OutputTerminal> getOutputTerminals() {
		return outputTerminals;
	}

	/* (non-Javadoc)
	 * @see org.mithrim.epf.core.EventProducer#setOutputTerminals(java.util.Set)
	 */
	@Override
	public void setOutputTerminals(final Set<OutputTerminal> outputTerminals) {
		this.outputTerminals = outputTerminals;
	}

	protected EventBuilderFactory getEventBuilderFactory() {
		return eventBuilderFactory;
	}

	public void setEventBuilderFactory(final EventBuilderFactory eventBuilderFactory) {
		this.eventBuilderFactory = eventBuilderFactory;
	}

	public boolean isOccurrenceTimeEnabled() {
		return occurrenceTimeEnabled;
	}

	void setOccurrenceTimeEnabled(final boolean occurrenceTimeEnabled) {
		this.occurrenceTimeEnabled = occurrenceTimeEnabled;
	}

	public boolean isValidationEnabled() {
		return validationEnabled;
	}

	void setValidationEnabled(final boolean validationEnabled) {
		this.validationEnabled = validationEnabled;
	}

	protected EventValidator getEventValidator() {
		return validator;
	}

	public void setEventValidator(final EventValidator validator) {
		this.validator = validator;
	}
}
