/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mithrim.epf.core.validation;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.mithrim.epf.core.event.Event;
import org.mithrim.epf.core.EventType;
import org.mithrim.epf.core.common.AttributeType;
import org.mithrim.epf.core.common.Occurrence;
import org.mithrim.epf.core.common.PayloadAttributeOccurrence;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * The <code>EventValidator</code> validates a given event instance against it's
 * event type definition. This should be primarily used during development, as it
 * has a significant performance overhead. During development, it is strongly
 * recommended to enable validation at least on <code>OutputTerminal</code>s to
 * find invalid events as early as possible.
 *
 * @author Dirk Jablonski
 */
public final class EventValidator {
	private static final Logger logger = LoggerFactory.getLogger(EventValidator.class);

	private transient Map<String, org.mithrim.epf.core.EventType> eventTypeMap;

	public boolean isValid(final Event event) {
		if (null == event) {
			logger.error("Illegal argument: event to validate must not be null");

			throw new IllegalArgumentException("event must not be null");
		}

		final EventType type = eventTypeMap.get(event.getTypeId());

		boolean isOk = checkHeaderAttributes(event, type);

		if (!isOk) {
			return false;
		}

		isOk = checkPayloadAttributes(event, type);

		if (!isOk) {
			return false;
		}

		isOk = checkOccurrence(event.getOpenContent(), type.getOpenContentIndicator(), "open content");

		if (!isOk) {
			return false;
		}

		// All checks are passed
		return true;
	}

	public void setEventTypes(final Set<EventType> eventTypes) {
		eventTypeMap = new HashMap<String, EventType>(eventTypes.size());

		for (EventType eventType : eventTypes) {
			eventTypeMap.put(eventType.getId(), eventType);
		}
	}

	/**
	 * @param event
	 */
	private boolean checkHeaderAttributes(final Event event, final EventType type) {
		boolean isOk = true;

		isOk &= checkOccurrence(event.getOccurrenceTime(), type.getOccurrenceTimeIndicator(), "occurrenceTime");
		isOk &= checkOccurrence(event.getCertainty(), type.getCertaintyIndicator(), "certainty");
		isOk &= checkOccurrence(event.getAnnotation(), type.getAnnotationIndicator(), "annotation");
		isOk &= checkOccurrence(event.getSource(), type.getSourceIndicator(), "source");

		return isOk;
	}

	/**
	 *
	 *
	 * @param event the event to check
	 * @param type the eventType to check against
	 * @return <code>true</code> if the payload attributes match the type definition,
	 *         <code>false</code> otherwise
	 */
	private boolean checkPayloadAttributes(final Event event, final EventType type) {
		boolean isOk = true;

		if (null != type.getPayloadAttributeTypes()) {
			for (AttributeType attribute : type.getPayloadAttributeTypes()) {
				isOk &= checkPayloadAttributeOccurrence(event.getPayloadAttribute(attribute.getName()), attribute.getOccurrence(), attribute.getName());
				isOk &= checkPayloadAttributeDataType(event.getPayloadAttribute(attribute.getName()), attribute.getDataType(), attribute.getName());
			}
		}

		return isOk;
	}

	/**
	 * @param value
	 * @param indicator
	 * @param messageText
	 */
	private boolean checkOccurrence(final Serializable value, final Occurrence indicator, final String attributeName) {
		if (null != value) {
			if (Occurrence.NOT_APPLICABLE == indicator) {
				logger.error("Event validation failed: header attribute {} is not allowed for this event type", attributeName);

				return false;
			}
		}
		else {
			if (Occurrence.REQUIRED == indicator) {
				logger.error("Event validation failed: header attribute {} is required for this event type", attributeName);

				return false;
			}
		}

		return true;
	}
	/**
	 * @param value
	 * @param indicator
	 * @param messageText
	 */
	private boolean checkPayloadAttributeOccurrence(final Serializable value, final PayloadAttributeOccurrence indicator, final String attributeName) {
		if (null == value && PayloadAttributeOccurrence.REQUIRED == indicator) {
			logger.error("Event validation failed: payload attribute {} is required for this event type", attributeName);

			return false;
		}

		return true;
	}

	/**
	 * @param value
	 * @param dataType
	 * @param attributeName
	 * @return
	 */
	private boolean checkPayloadAttributeDataType(final Serializable value, final String dataType, final String attributeName) {
		if (null != value) {
			try {
				// FIXME Do not use Class.forName(), as it is not compatible with OSGi
				final Class<?> dataTypeClazz = Class.forName(dataType);

				if (!dataTypeClazz.isAssignableFrom(value.getClass())) {
					logger.error("Event validation failed: payload attribute {} requires data type {} but has {}", new Object[] {attributeName, dataType, value.getClass().getName()});

					return false;
				}
			}
			catch (final ClassNotFoundException e) {
				final String message = String.format("Event validation failed: Unable to find defined data type class {} for payload attribute {}", dataType, attributeName);
				logger.error(message , e);

				return false;
			}
		}

		return true;
	}

}
